﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Edge.Core.Domain.FccOrderInfo
{
    [Table("qr_order")]
    public class FccOrderInfo
    {
        /// <summary>
        /// 订单id
        /// </summary>
        [Key]
        [DatabaseGenerated(DatabaseGeneratedOption.Identity)]
        public long Id { get; set; }

        /// <summary>
        /// 流水号
        /// </summary>
        public int Ttc { get; set; }

        /// <summary>
        /// 授权时间
        /// </summary>
        public DateTime AuthorizationTime { get; set; }

        /// <summary>
        /// 挂枪时间
        /// </summary>
        public DateTime? EndTime { get; set; }

        /// <summary>
        /// 交易时间
        /// </summary>
        public DateTime? PaymentTime { get; set; }

        /// <summary>
        /// 枪号
        /// </summary>
        public int NozzleNum { get; set; }

        /// <summary>
        /// 油品名
        /// </summary>
        public String OilName { get; set; }

        /// <summary>
        /// 支付状态： 0：未支付 1:已支付 2：退全款 3：退余额：4：卡支付
        /// </summary>
        public int PaymentStatus {  get; set; }

        /// <summary>
        /// 支付类型
        /// </summary>
        public int? PayType { get; set; }

        /// <summary>
        /// 云端id
        /// </summary>
        public long? CloundOrderId { get; set; }

        /// <summary>
        /// 实际加油金额
        /// </summary>
        public decimal Amount { get; set; }

        /// <summary>
        /// 实际加油升数
        /// </summary>
        public decimal Volume { get; set; }

        /// <summary>
        /// 实际支付金额
        /// </summary>
        public decimal? AmountPayable { get; set; }

        /// <summary>
        /// 退款金额
        /// </summary>
        public decimal? RefundAmount { get; set; }
        /// <summary>
        /// 单价
        /// </summary>
        public decimal Price { get; set; }

        /// <summary>
        /// 是否已上传云端—— 0：未上传；1：已上传；2：不需要上传
        /// </summary>
        public int UploadState { get; set; }

        /// <summary>
        /// 是否删除——0：否；1：是
        /// </summary>
        public int IsDelete { get; set; }

        /// <summary>
        /// 支付用户名
        /// </summary>
        public string UserName { get; set; }

        /// <summary>
        /// 手机号
        /// </summary>
        public string PhoneNumber { get; set; }

        /// <summary>
        /// 支付类型名称
        /// </summary>
        public string PaymentName { get; set; }

        /// <summary>
        /// 泵码
        /// </summary>
        public decimal? PumpCode { get; set; }

        /// <summary>
        /// 授权状态：0：未授权；1：已授权;3:加油中；4：订单完成
        /// </summary>
        public int AuthorizationStatus { get; set; } = 0;

        /// <summary>
        /// 退款状态：0：无退款；1：全额退款；2：部分退款
        /// </summary>
        public int RefundStatus { get; set; } = 0;

        /// <summary>
        /// 预定升数
        /// </summary>
        public decimal? VolumePayable { get; set; }
    }

}
