﻿using System.Numerics;

namespace Edge.Core.Domain.FccStationInfo.Output
{
    /// <summary>
    /// 获取站点基本信息结果
    /// </summary>
    public class StationInfoOutput
    {
        /// <summary>
        /// 总数量
        /// </summary>
        public int Total { get; set; }

        /// <summary>
        /// 站点信息
        /// </summary>
        public List<StationInfo> StationList { get; set; }
    }

    public class StationInfo
    {
        //public StationInfo() { }

        public StationInfo(FccStationInfo fccStationInfo)
        {
            this.Id = fccStationInfo.Id;
            this.BuildId = fccStationInfo.BuildId;
            this.SecretId = fccStationInfo.SecretId;
            this.AcessKey = fccStationInfo.AcessKey;
            this.Name = fccStationInfo.Name;
            this.Longitude = fccStationInfo.Longitude;
            this.Latitude = fccStationInfo.Latitude;
            this.SmallProgram = fccStationInfo.SmallProgram;
            this.CloudService = fccStationInfo.CloudService;
            this.MqttService = fccStationInfo.MqttService;
            this.IcardService = fccStationInfo.IcardService;
            this.WebSocketPort = fccStationInfo.WebSocketPort;
            this.PaymentType = fccStationInfo.PaymentType;
            this.CheckOrderInterval = fccStationInfo.CheckOrderInterval;
        }
        /// <summary>
        /// id
        /// </summary>
        public long? Id { get; set; }

        /// <summary>
        /// 云端站点 id
        /// </summary>
        public string BuildId { get; set; }

        /// <summary>
        /// 客户端id
        /// </summary>
        public string SecretId { get; set; }

        /// <summary>
        /// 与云端加密的 key
        /// </summary>
        public string AcessKey { get; set; }

        /// <summary>
        /// 站点名称
        /// </summary>
        public string? Name { get; set; }

        /// <summary>
        /// 经度
        /// </summary>
        public decimal Longitude { get; set; }

        /// <summary>
        /// 纬度
        /// </summary>
        public decimal Latitude { get; set; }

        /// <summary>
        /// 小程序链接地址
        /// </summary>
        public string SmallProgram { get; set; }

        /// <summary>
        /// 云端地址
        /// </summary>
        public string CloudService { get; set; }

        /// <summary>
        /// 云端  MQTT 地址
        /// </summary>
        public string MqttService { get; set; }

        /// <summary>
        /// icard 数据库连接地址
        /// </summary>
        public string? IcardService { get; set; }

        /// <summary>
        /// 本地 fcc 与 fcc 页面 webSocket 端口
        /// </summary>
        public string WebSocketPort { get; set; }

        /// <summary>
        /// 支付方式，【id+支付名称,id+支付名称】
        /// </summary>
        public string PaymentType { get; set; }

        /// <summary>
        /// 检查订单时间间隔
        /// </summary>
        public int CheckOrderInterval { get; set; }
    }
}
